type KindOfClass<T = unknown> = new (...args: any[]) => T;

interface KindOfImmutableEntity<T = unknown> extends KindOfClass<T> {
}
declare function ImmutableEntity(): ClassDecorator;

interface CamelToSnakeOptions {
    camelToSnakeCase: true;
    snakeToCamelCase?: false;
}
interface SnakeToCamelOptions {
    camelToSnakeCase?: false;
    snakeToCamelCase: true;
}
type ImmutableEntityOptions = CamelToSnakeOptions | SnakeToCamelOptions;

declare class ImmutableEntityBuilder<T extends KindOfImmutableEntity> {
    private readonly Type;
    private readonly data;
    private options;
    constructor(Type: T, data: Record<string, unknown>);
    build(): InstanceType<T>;
    withOptions(options: ImmutableEntityOptions): this;
    withProperty<V>(key: keyof InstanceType<T>, value: V, applyDecorators?: boolean): this;
    private processData;
    private getDefaults;
    private transformProperty;
}

declare class ImmutableEntityManager<T extends KindOfImmutableEntity> {
    private readonly Type;
    constructor(Type: T);
    static getUniqueManager<T extends KindOfImmutableEntity>(Type: T): ImmutableEntityManager<T>;
    parseFromJson(data: Record<string, unknown>): ImmutableEntityBuilder<T>;
    getEmptyBuilder(): ImmutableEntityBuilder<T>;
}

/**
 *
 * @param value
 * @param strict – Should be type checking
 * @constructor
 */
declare function ImmutableEntityDefaultValue<T>(value: T): PropertyDecorator;

declare function ImmutableEntityTyped<T>(Type: KindOfClass<T>): PropertyDecorator;

interface ImmutableEntityTransformOptions {
    beforeTyped?: boolean;
    afterTyped?: boolean;
}
declare function ImmutableEntityTransform(options?: ImmutableEntityTransformOptions): PropertyDecorator;

declare function ImmutableEntityTransient(): PropertyDecorator;

declare function camelToSnake(str: string | symbol): string | symbol;

declare function snakeToCamel(str: string | symbol): string | symbol;

export { ImmutableEntity, ImmutableEntityBuilder, ImmutableEntityDefaultValue, ImmutableEntityManager, ImmutableEntityTransform, type ImmutableEntityTransformOptions, ImmutableEntityTransient, ImmutableEntityTyped, type KindOfImmutableEntity, camelToSnake, snakeToCamel };
