var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

// src/core/immutable-entity.ts
import "reflect-metadata";

// src/core/immutable-entity-constructor.ts
function assignImmutableEntityProperties(instance, data) {
  Object.entries(data).forEach(
    ([property, value]) => Object.defineProperty(instance, property, {
      value
    })
  );
}
function buildImmutableEntityConstructor(OriginalClass) {
  return function(...args) {
    const data = args[0];
    const instance = new OriginalClass();
    assignImmutableEntityProperties(instance, data);
    return instance;
  };
}

// src/core/immutable-entity.ts
function ImmutableEntity() {
  return function(target) {
    const OriginalClass = target;
    const immutableEntityConstructor = buildImmutableEntityConstructor(OriginalClass);
    immutableEntityConstructor.prototype = OriginalClass.prototype;
    Reflect.defineMetadata("design:immutable-entity" /* ImmutableEntity */, true, immutableEntityConstructor);
    return immutableEntityConstructor;
  };
}

// src/utils/camel-to-snake.ts
function camelToSnake(str) {
  if (typeof str === "string") {
    return str.replace(/[A-Z]/g, (letter) => `_${letter.toLowerCase()}`);
  }
  return str;
}

// src/utils/snake-to-camel.ts
function snakeToCamel(str) {
  if (typeof str === "string") {
    return str.toLowerCase().replace(
      /([-_][a-z])/g,
      (group) => group.toUpperCase().replace("-", "").replace("_", "")
    );
  }
  return str;
}

// src/core/immutable-entity-builder.ts
var ImmutableEntityBuilder = class _ImmutableEntityBuilder {
  constructor(Type, data) {
    this.options = {
      snakeToCamelCase: true,
      camelToSnakeCase: false
    };
    this.Type = Type;
    this.data = Object.entries(data).reduce((acc, [property, value]) => __spreadProps(__spreadValues({}, acc), {
      [property]: {
        value,
        applyDecorators: true
      }
    }), {});
  }
  build() {
    return new this.Type(this.processData(this.data));
  }
  withOptions(options) {
    this.options = options;
    return this;
  }
  withProperty(key, value, applyDecorators = false) {
    this.data[key] = {
      value,
      applyDecorators
    };
    return this;
  }
  processData(data) {
    const transientProperties = Reflect.getMetadata("design:transients" /* Transients */, this.Type.prototype);
    const typedProperties = Reflect.getMetadata("design:typed" /* Typed */, this.Type.prototype);
    return Object.entries(data).filter(([property, _]) => transientProperties instanceof Map ? transientProperties.has(property) : true).concat(Object.entries(this.getDefaults())).reduce((acc, [property, value]) => this.transformProperty(property, value.value, value.applyDecorators, typedProperties, acc), {});
  }
  getDefaults() {
    const defaults = Reflect.getMetadata("design:defaults" /* Defaults */, this.Type.prototype);
    return typeof defaults === "object" ? Array.from(defaults.entries()).reduce((acc, [property, value]) => __spreadProps(__spreadValues({}, acc), {
      [property]: {
        value,
        applyDecorators: false
      }
    }), {}) : {};
  }
  transformProperty(property, value, applyDecorators, typedProperties, acc) {
    let transformedPropertyName = property;
    let nextValue = value;
    if (!applyDecorators) {
      return __spreadProps(__spreadValues({}, acc), {
        [transformedPropertyName]: nextValue
      });
    }
    if (this.options.camelToSnakeCase === true) {
      transformedPropertyName = camelToSnake(property);
    } else if (this.options.snakeToCamelCase) {
      transformedPropertyName = snakeToCamel(property);
    }
    if ((typedProperties == null ? void 0 : typedProperties.has(property)) === true) {
      const isImmutableEntity = Reflect.getMetadata("design:immutable-entity" /* ImmutableEntity */, typedProperties.get(property));
      const PropertyType = typedProperties.get(property);
      if (isImmutableEntity === true && typeof value === "object") {
        const propertyBuilder = new _ImmutableEntityBuilder(PropertyType, value).withOptions(this.options);
        nextValue = propertyBuilder.build();
      } else {
        nextValue = new PropertyType(value);
      }
    }
    return __spreadProps(__spreadValues({}, acc), {
      [transformedPropertyName]: nextValue
    });
  }
};

// src/core/immutable-entity-manager.ts
var ImmutableEntityManager = class _ImmutableEntityManager {
  constructor(Type) {
    this.Type = Type;
  }
  static getUniqueManager(Type) {
    return new _ImmutableEntityManager(Type);
  }
  parseFromJson(data) {
    return new ImmutableEntityBuilder(this.Type, data);
  }
  getEmptyBuilder() {
    return new ImmutableEntityBuilder(this.Type, {});
  }
};

// src/decorators/default.ts
function ImmutableEntityDefaultValue(value) {
  return function(target, propertyKey) {
    var _a;
    const defaults = (_a = Reflect.getMetadata("design:defaults" /* Defaults */, target)) != null ? _a : /* @__PURE__ */ new Map();
    defaults.set(camelToSnake(propertyKey), value);
    defaults.set(snakeToCamel(propertyKey), value);
    Reflect.defineMetadata("design:defaults" /* Defaults */, defaults, target);
  };
}

// src/decorators/typed.ts
function ImmutableEntityTyped(Type) {
  return function(target, propertyKey) {
    var _a;
    const typedProperties = (_a = Reflect.getMetadata("design:typed" /* Typed */, target)) != null ? _a : /* @__PURE__ */ new Map();
    typedProperties.set(camelToSnake(propertyKey), Type);
    typedProperties.set(snakeToCamel(propertyKey), Type);
    Reflect.defineMetadata("design:typed" /* Typed */, typedProperties, target);
  };
}

// src/decorators/transform.ts
function ImmutableEntityTransform(options = { beforeTyped: true, afterTyped: false }) {
  return function(target, propertyKey) {
  };
}

// src/decorators/transient.ts
function ImmutableEntityTransient() {
  return function(target, propertyKey) {
    var _a;
    const transients = (_a = Reflect.getMetadata("design:transients" /* Transients */, target)) != null ? _a : /* @__PURE__ */ new Set();
    transients.add(camelToSnake(propertyKey)).add(snakeToCamel(propertyKey));
    Reflect.defineMetadata("design:transients" /* Transients */, transients, target);
  };
}
export {
  ImmutableEntity,
  ImmutableEntityBuilder,
  ImmutableEntityDefaultValue,
  ImmutableEntityManager,
  ImmutableEntityTransform,
  ImmutableEntityTransient,
  ImmutableEntityTyped,
  camelToSnake,
  snakeToCamel
};
//# sourceMappingURL=index.mjs.map